package rition.backend.aop.advice;

import jakarta.annotation.Nonnull;
import lombok.extern.slf4j.Slf4j;
import org.springframework.core.MethodParameter;
import org.springframework.http.MediaType;
import org.springframework.http.converter.HttpMessageConverter;
import org.springframework.http.server.ServerHttpRequest;
import org.springframework.http.server.ServerHttpResponse;
import org.springframework.web.bind.annotation.ControllerAdvice;
import org.springframework.web.servlet.mvc.method.annotation.ResponseBodyAdvice;
import rition.backend.annotation.WithRequestIdResponse;
import rition.backend.api.v1.dto.response.Response;
import rition.backend.api.v1.dto.response.ResponseWithRequestId;

import java.util.Objects;

/**
 * 处理带有@WithRequestIdResponse注释的接口，在返回响应时，如果有RequestId，则将其放入响应的json中。
 * 一般来说，在执行该advice的时候，通常也意味着requestId已经被设置
 */
@Slf4j
@ControllerAdvice
public class RequestIdResponseAdvice implements ResponseBodyAdvice<Object> {
    @Override
    public boolean supports(@Nonnull MethodParameter returnType,
                            @Nonnull Class<? extends HttpMessageConverter<?>> converterType) {
        var targetMethod = Objects.requireNonNull(returnType.getMethod());

        return targetMethod.getAnnotation(WithRequestIdResponse.class) != null ||
                targetMethod.getDeclaringClass().getAnnotation(WithRequestIdResponse.class) != null;
    }

    @Override
    public Object beforeBodyWrite(Object body, @Nonnull MethodParameter returnType,
                                  @Nonnull MediaType selectedContentType, @Nonnull Class selectedConverterType,
                                  @Nonnull ServerHttpRequest serverHttpRequest,
                                  @Nonnull ServerHttpResponse serverHttpResponse) {
        if (body == null) {
            return null;
        }

        if (body instanceof ResponseWithRequestId<?>) {
            return body;
        }

        if (body instanceof Response<?> response) {
            var requestIdHeaderList = serverHttpResponse.getHeaders().get("X-Request-ID");

            if (requestIdHeaderList == null || requestIdHeaderList.isEmpty()) {
                return body;
            }

            var responseWithResponseId = new ResponseWithRequestId<>(response);
            responseWithResponseId.setRequestId(requestIdHeaderList.get(0));
            return responseWithResponseId;
        }

        return body;
    }
}
