package rition.backend.api.interceptor;

import jakarta.annotation.Nonnull;
import jakarta.servlet.http.HttpServletRequest;
import jakarta.servlet.http.HttpServletResponse;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Component;
import org.springframework.web.method.HandlerMethod;
import org.springframework.web.servlet.HandlerInterceptor;
import rition.backend.annotation.WithRequestIdResponse;
import rition.backend.util.ObjectUtils;
import rition.backend.util.StringUtils;

import java.util.UUID;

/**
 * ResponseId的拦截器，为当前请求生成一个RequestId，并将其放入HttpServletRequest中的Attribute上下文中，
 * 同时设置HttpServletResponse的Header
 */
@Slf4j
@Component
public class ResponseIdInterceptor implements HandlerInterceptor {

    @Override
    public boolean preHandle(@Nonnull HttpServletRequest request,
                             @Nonnull HttpServletResponse response,
                             @Nonnull Object handler) {

        if (!(handler instanceof HandlerMethod handlerMethod)) {
            return true;
        }

        var methodAnnotation = handlerMethod.getMethodAnnotation(WithRequestIdResponse.class);
        var classAnnotation = handlerMethod
                .getMethod()
                .getDeclaringClass()
                .getAnnotation(WithRequestIdResponse.class);

        if (ObjectUtils.atLeaseOneNonNull(methodAnnotation, classAnnotation)) {
            var existsRequestId = response.getHeader("X-Request-ID");
            if (StringUtils.isEmpty(existsRequestId)) {
                var requestId = UUID.randomUUID().toString();
                response.setHeader("X-Request-ID", requestId);
                request.setAttribute("requestId", requestId);
            }
        }

        return true;
    }
}
