package rition.backend.api.resolver;

import jakarta.annotation.Nonnull;
import jakarta.servlet.http.HttpServletRequest;
import jakarta.servlet.http.HttpServletResponse;
import lombok.extern.slf4j.Slf4j;
import org.springframework.core.MethodParameter;
import org.springframework.stereotype.Component;
import org.springframework.web.bind.support.WebDataBinderFactory;
import org.springframework.web.context.request.NativeWebRequest;
import org.springframework.web.method.support.HandlerMethodArgumentResolver;
import org.springframework.web.method.support.ModelAndViewContainer;
import rition.backend.annotation.paramter.RequestId;

import java.util.UUID;

/**
 * 自动将request注入到controller的方法参数中，使用@RequestId标注后，将RequestId注入到参数中
 */
@Slf4j
@Component
public class RequestIdArgumentResolver implements HandlerMethodArgumentResolver {
    @Override
    public boolean supportsParameter(MethodParameter parameter) {
        return parameter.hasParameterAnnotation(RequestId.class);
    }

    @Override
    public Object resolveArgument(@Nonnull MethodParameter parameter, ModelAndViewContainer mavContainer,
                                  @Nonnull NativeWebRequest webRequest, WebDataBinderFactory binderFactory) {

        var request = webRequest.getNativeRequest(HttpServletRequest.class);
        var response = webRequest.getNativeResponse(HttpServletResponse.class);
        assert request != null;
        assert response != null;

        // 如果Attribute上下文和ResponseHeader都有RequestId，则现在生成一个新的
        // 相应的方法已经使用@WithRequestIdResponse注解在拦截器中设置，
        // 则其实可以断言Attribute上下文和ResponseHeader都有RequestId
        String requestId = (String) request.getAttribute("requestId");
        if (requestId == null) {
            requestId = response.getHeader("X-Request-ID");
            if (requestId == null) {
                requestId = UUID.randomUUID().toString();
                response.setHeader("X-Request-ID", requestId);
            }

            request.setAttribute("requestId", requestId);
            return requestId;
        } else {
            if (response.getHeader("X-Request-ID") == null) {
                response.setHeader("X-Request-ID", requestId);
            }
        }

        return requestId;
    }
}
