package rition.backend.service;

import lombok.extern.slf4j.Slf4j;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Service;
import rition.common.data.dto.MetricDataDto;
import rition.common.data.dto.log.RequestProcessTraceRecord;
import rition.service.collector.MetricCollectingService;

import java.time.Duration;
import java.time.temporal.ChronoUnit;
import java.util.Map;

/**
 * MetricCollectorService 指标数据收集类，负责收集来自云主机探针上报的数据，并将其入库等，
 * 这里只是个代理类，实际的实现在collector模块，
 * 这样做只是为了方便日后将collector等模块独立拆分出来，
 * 这种情况下实际上数据收集的入口应当直接放在collector里，而不是api部分
 */
@Slf4j
@Service
public class MetricDataCollectingService {

    private final MetricCollectingService metricCollectingService;
    private final RedisTemplate<String, Object> redisTemplate;

    public MetricDataCollectingService(MetricCollectingService metricCollectingService,
                                       RedisTemplate<String, Object> redisTemplate) {
        this.metricCollectingService = metricCollectingService;
        this.redisTemplate = redisTemplate;
    }

    private static final String PROCESS_TRACE_REDIS_KEY_FORMAT = "rition:request:trace:%s";

    /**
     * 接收处理好的监控指标数据，异步处理，请求发送后，处理状态使用requestId追踪，处理结果放置于redis
     *
     * @param requestId           此次的请求id，用于追踪处理结果
     * @param collectedMetricData 监控指标数据，protobuf对象
     */
    @Async
    public void receiveData(String requestId, Map<Long, Map<String, MetricDataDto>> collectedMetricData) {
        RequestProcessTraceRecord record;
        try {
            var start = System.currentTimeMillis();
            metricCollectingService.receiveData(collectedMetricData);
            var cost = System.currentTimeMillis() - start;
            record = RequestProcessTraceRecord.success(requestId, "ok", cost, null);
        } catch (Exception e) {
            record = RequestProcessTraceRecord.builder()
                    .exception(e)
                    .requestId(requestId)
                    .message("process failed: " + e)
                    .status(RequestProcessTraceRecord.OperationStatus.FAIL)
                    .build();
        }

        try {
            redisTemplate.opsForValue().set(PROCESS_TRACE_REDIS_KEY_FORMAT.formatted(requestId), record);
        } catch (Exception e2) {
            log.error("请求结果保存出现异常: ", e2);
            log.error("请求结果：{}", record);
        }
    }
}
